function [descriptors] = sift_computedescriptors(input_image, features)
%SIFT_COMPUTEDESCRIPTORS Generate sift descriptors
%   [DESCRIPTORS] = SIFT_COMPUTEDESCRIPTORS(INPUT_IMAGE, FEATURES) takes as input a grayscale image 
%   and locations/scales/orientations within that image that represent salient points. The 
%   function returns sift descriptors for each of the points.
%
%   input parameters:
%       INPUT_IMAGE - PxQ matrix representing an image.
%       FEATURES    - 4xF matrix. Each column contains x-location, y-location, scale and 
%                     orientation.
%
%   output parameters:
%       DESCRIPTORS - 128xF matrix. Each column is a descriptor.
%

if ~exist('siftdescriptor.m', 'file')
    addpath('sift');
end

% Lowe's equivalents choices
[M, N, ~] = size(input_image);
S         =  3;
omin      = -1;
O         = floor(log2(min(M, N))) - omin - 3; % up to 8x8 images
sigma0    = 1.6 * 2 ^ (1 / S);                 % smooth lev. -1 at 1.6
sigman    = 0.5;
NBP       = 4;
NBO       = 8;
magnif    = 3.0;

% Compute scale-space images
gss = gaussianss(input_image, sigman, O, S, omin, -1, S + 1, sigma0);

% Compute octave
o = round(log2(features(3, :)) + 1);

% Get oframes
oframes(1, :) = features(1, :) ./ (2 .^ (o - 1 + gss.omin));
oframes(2, :) = features(2, :) ./ (2 .^ (o - 1 + gss.omin));
oframes(3, :) = gss.S * log2(features(3, :) ./ (2 .^ (o - 1 + gss.omin) * gss.sigma0));
oframes(4, :) = features(4, :);

% Compute descriptors
descriptors = zeros(128, size(features, 2));
for current_o = 1:size(gss.octave, 2)
  indices = (o == current_o);
  
  sh = siftdescriptor(gss.octave{current_o}, oframes(:,indices), gss.sigma0, gss.S, gss.smin, ...
                      'Magnif', magnif, 'NumSpatialBins', NBP, 'NumOrientBins', NBO);
  
  descriptors(:, indices) = sh;
end

end